@@ -6,6 +6,7 @@ import errno
 import re
 import socket
 import ssl
+from ssl import SSLError
 from contextlib import contextmanager
 
 from .exceptions import UnexpectedFrame
@@ -13,11 +14,6 @@ from .five import items
 from .platform import KNOWN_TCP_OPTS, SOL_TCP, pack, unpack
 from .utils import get_errno, set_cloexec
 
-try:
-    from ssl import SSLError
-except ImportError:  # pragma: no cover
-    class SSLError(Exception):  # noqa
-        """Dummy SSL exception."""
 
 _UNAVAIL = {errno.EAGAIN, errno.EINTR, errno.ENOENT, errno.EWOULDBLOCK}
 
@@ -112,6 +108,10 @@ class _AbstractTransport(object):
                     # Non-blocking SSL sockets can throw SSLError
                     raise socket.timeout()
                 raise
+            except socket.error as exc:
+                if get_errno(exc) == errno.EWOULDBLOCK:
+                    raise socket.timeout()
+                raise
             finally:
                 if timeout != prev:
                     sock.settimeout(prev)
@@ -180,9 +180,11 @@ class _AbstractTransport(object):
         for timeout, interval in ((socket.SO_SNDTIMEO, write_timeout),
                                   (socket.SO_RCVTIMEO, read_timeout)):
             if interval is not None:
+                sec = int(interval)
+                usec = int((interval - sec) * 1000000)
                 self.sock.setsockopt(
                     socket.SOL_SOCKET, timeout,
-                    pack('ll', interval, 0),
+                    pack('ll', sec, usec),
                 )
         self._setup_transport()
 
@@ -255,7 +257,7 @@ class _AbstractTransport(object):
             if size > SIGNED_INT_MAX:
                 part1 = read(SIGNED_INT_MAX)
                 part2 = read(size - SIGNED_INT_MAX)
-                payload = ''.join([part1, part2])
+                payload = b''.join([part1, part2])
             else:
                 payload = read(size)
             read_frame_buffer += payload
@@ -315,7 +317,7 @@ class SSLTransport(_AbstractTransport):
 
     def _wrap_socket_sni(self, sock, keyfile=None, certfile=None,
                          server_side=False, cert_reqs=ssl.CERT_NONE,
-                         ca_certs=None, do_handshake_on_connect=True,
+                         ca_certs=None, do_handshake_on_connect=False,
                          suppress_ragged_eofs=True, server_hostname=None,
                          ciphers=None, ssl_version=None):
         """Socket wrap with SNI headers.
@@ -323,24 +325,31 @@ class SSLTransport(_AbstractTransport):
         Default `ssl.wrap_socket` method augmented with support for
         setting the server_hostname field required for SNI hostname header
         """
-        opts = dict(sock=sock, keyfile=keyfile, certfile=certfile,
-                    server_side=server_side, cert_reqs=cert_reqs,
-                    ca_certs=ca_certs,
-                    do_handshake_on_connect=do_handshake_on_connect,
-                    suppress_ragged_eofs=suppress_ragged_eofs,
-                    ciphers=ciphers)
         # Setup the right SSL version; default to optimal versions across
         # ssl implementations
-        if ssl_version is not None:
-            opts['ssl_version'] = ssl_version
-        else:
+        if ssl_version is None:
             # older versions of python 2.7 and python 2.6 do not have the
             # ssl.PROTOCOL_TLS defined the equivalent is ssl.PROTOCOL_SSLv23
             # we default to PROTOCOL_TLS and fallback to PROTOCOL_SSLv23
+            # TODO: Drop this once we drop Python 2.7 support
             if hasattr(ssl, 'PROTOCOL_TLS'):
-                opts['ssl_version'] = ssl.PROTOCOL_TLS
+                ssl_version = ssl.PROTOCOL_TLS
             else:
-                opts['ssl_version'] = ssl.PROTOCOL_SSLv23
+                ssl_version = ssl.PROTOCOL_SSLv23
+
+        opts = {
+            'sock': sock,
+            'keyfile': keyfile,
+            'certfile': certfile,
+            'server_side': server_side,
+            'cert_reqs': cert_reqs,
+            'ca_certs': ca_certs,
+            'do_handshake_on_connect': do_handshake_on_connect,
+            'suppress_ragged_eofs': suppress_ragged_eofs,
+            'ciphers': ciphers,
+            'ssl_version': ssl_version
+        }
+
         sock = ssl.wrap_socket(**opts)
         # Set SNI headers if supported
         if (server_hostname is not None) and (
@@ -348,19 +357,17 @@ class SSLTransport(_AbstractTransport):
                 hasattr(ssl, 'SSLContext')):
             context = ssl.SSLContext(opts['ssl_version'])
             context.verify_mode = cert_reqs
-            context.check_hostname = True
-            context.load_cert_chain(certfile, keyfile)
+            if cert_reqs != ssl.CERT_NONE:
+                context.check_hostname = True
+            if (certfile is not None) and (keyfile is not None):
+                context.load_cert_chain(certfile, keyfile)
             sock = context.wrap_socket(sock, server_hostname=server_hostname)
         return sock
 
     def _shutdown_transport(self):
-        """Unwrap a Python 2.6 SSL socket, so we can call shutdown()."""
+        """Unwrap a SSL socket, so we can call shutdown()."""
         if self.sock is not None:
-            try:
-                unwrap = self.sock.unwrap
-            except AttributeError:
-                return
-            self.sock = unwrap()
+            self.sock = self.sock.unwrap()
 
     def _read(self, n, initial=False,
               _errnos=(errno.ENOENT, errno.EAGAIN, errno.EINTR)):
@@ -374,6 +381,10 @@ class SSLTransport(_AbstractTransport):
                 try:
                     s = recv(n - len(rbuf))  # see note above
                 except socket.error as exc:
+                    # ssl.sock.read may cause a SSLerror without errno
+                    # http://bugs.python.org/issue10272
+                    if isinstance(exc, SSLError) and 'timed out' in str(exc):
+                        raise socket.timeout()
                     # ssl.sock.read may cause ENOENT if the
                     # operation couldn't be performed (Issue celery#1414).
                     if exc.errno in _errnos:
@@ -396,14 +407,11 @@ class SSLTransport(_AbstractTransport):
         while s:
             try:
                 n = write(s)
-            except (ValueError, AttributeError):
+            except ValueError:
                 # AG: sock._sslobj might become null in the meantime if the
                 # remote connection has hung up.
-                # In python 3.2, an AttributeError is raised because the SSL
-                # module tries to access self._sslobj.write (w/ self._sslobj ==
-                # None)
                 # In python 3.4, a ValueError is raised is self._sslobj is
-                # None. So much for portability... :/
+                # None.
                 n = 0
             if not n:
                 raise IOError('Socket closed')
