@@ -8,7 +8,8 @@ from case import ContextMock, Mock, call, patch
 
 from amqp import Connection, spec
 from amqp.connection import SSLError
-from amqp.exceptions import ConnectionError, NotFound, ResourceError
+from amqp.exceptions import (ConnectionError, NotFound,
+                             RecoverableConnectionError, ResourceError)
 from amqp.five import items
 from amqp.sasl import AMQPLAIN, EXTERNAL, GSSAPI, PLAIN, SASL
 from amqp.transport import TCPTransport
@@ -356,6 +357,12 @@ class test_Connection:
         c2 = self.conn.channel(3, callback)
         assert c2 is c
 
+    def test_channel_when_connection_is_closed(self):
+        self.conn.collect()
+        callback = Mock(name='callback')
+        with pytest.raises(RecoverableConnectionError):
+            self.conn.channel(3, callback)
+
     def test_is_alive(self):
         with pytest.raises(NotImplementedError):
             self.conn.is_alive()
@@ -417,6 +424,11 @@ class test_Connection:
             (50, 60), 'payload', 'content',
         )
 
+    def test_on_inbound_method_when_connection_is_closed(self):
+        self.conn.collect()
+        with pytest.raises(RecoverableConnectionError):
+            self.conn.on_inbound_method(1, (50, 60), 'payload', 'content')
+
     def test_close(self):
         self.conn.collect = Mock(name='collect')
         self.conn.close(reply_text='foo', method_sig=spec.Channel.Open)
